/*

This file is part of the Physical Audio Digital Filters (PADF)

PADF can be obtained at http://stefanofasciani.com/padf.html
TSAM Copyright (C) 2017 Stefano Fasciani, University of Wollongong
Inquiries: stefanofasciani@stefanofasciani.com

The PADF is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

The PADF is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Less General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with TSAM. If not, see <http://www.gnu.org/licenses/>.

If you use the PADF or any part of it in any program or publication, please acknowledge its authors by adding a reference to this pubblication:

S. Fasciani, 2017, "Pysical Audio Digital Filters" in proceedings of the 17th Conference on New Interfaces for Musical Expression, Copenhagen, Denmark

*/

inlets = 1;
outlets = 0;


var threshold = 0.3;

function msg_float(a){
	threshold = a;
}


function bang() {
	
	var buffer = new Buffer("ir1");
	var framecount = buffer.framecount()
	var source = buffer.peek(1, 0, framecount);
	var env = new Array();
	var delta = new Array();

	// RMS Envelope Analysis
	var feedback_coefficient = 0.999;
	var feedback_sample = 0.0;

	for (var i=0; i<framecount; ++i) {
		var source_sample = source[i];
		var squared = source_sample * source_sample;
		var mean = feedback_sample * feedback_coefficient + squared * (1.0-feedback_coefficient);
		var root = Math.sqrt(mean);

		env[i] = root;
		feedback_sample = mean;
	}

	// Delta Analysis
	var max = 0.0;
	for (var i=0; i<framecount; ++i) {	
		if (i==0)
			delta[i] = 0.0;
		else
			delta[i] = env[i] - env[i-1];
		
		if (delta[i] > max)
			max = delta[i];
	}
	
	// normalize
	var scale = 1.0/max;

	for (var i=0; i<framecount; ++i) {
		delta[i] = delta[i] * scale;
	}
	
	var outvalid = 0;
	var offset;
	// threshold analysis & crop
	for (var i=0; i<framecount; ++i) {
		if (delta[i] > threshold) {
			offset = i -1;
			var cropped = buffer.peek(1, offset, (framecount-offset));
			var zeropad = new Array();
			buffer.poke(1, 0, cropped);
			for (var i=0; i<offset; ++i) zeropad[i]=0;
			buffer.poke(1, (framecount-offset), zeropad);
			break;
		}
	}
	

	var buffer = new Buffer("ir2");
	var framecount = buffer.framecount()
	var source = buffer.peek(1, 0, framecount);
	var env = new Array();
	var delta = new Array();

	// RMS Envelope Analysis
	var feedback_coefficient = 0.999;
	var feedback_sample = 0.0;

	for (var i=0; i<framecount; ++i) {
		var source_sample = source[i];
		var squared = source_sample * source_sample;
		var mean = feedback_sample * feedback_coefficient + squared * (1.0-feedback_coefficient);
		var root = Math.sqrt(mean);

		env[i] = root;
		feedback_sample = mean;
	}

	// Delta Analysis
	var max = 0.0;
	for (var i=0; i<framecount; ++i) {	
		if (i==0)
			delta[i] = 0.0;
		else
			delta[i] = env[i] - env[i-1];
		
		if (delta[i] > max)
			max = delta[i];
	}
	
	// normalize
	var scale = 1.0/max;

	for (var i=0; i<framecount; ++i) {
		delta[i] = delta[i] * scale;
	}

	var outvalid = 0;
	var offset;
	// threshold analysis & crop
	for (var i=0; i<framecount; ++i) {
		if (delta[i] > threshold) {	
			offset = i-1;
			var cropped = buffer.peek(1, offset, (framecount-offset));
			var zeropad = new Array();
			buffer.poke(1, 0, cropped);
			for (var i=0; i<offset; ++i) zeropad[i]=0;
			buffer.poke(1, (framecount-offset), zeropad);
			break;
		}
	}
	
}
